function rc=contagion(dmat,blobs)

% function rc=contagion(dmat,blobs)
%
% Function to calculate the contagion of a 3D matrix, dmat, having n blob
% types specified in the vector blobs. Blob types must be integers: 1
% 2 3 ...
%
% This version uses a Moore neighbourhood.
%
% Written by Lael Parrott, October 2004
%
% Based on the formulas supplied by Li, H & Reynolds, J.F. 1993.  A new
% contagion index to quantify spatial patterns of landscapes. Landscape
% Ecology 8(3):155-162.

% Distributed as part of the 3D Metrics Toolbox

% Licensed under a Creative Commons Attribution-NonCommercial-Share Alike
% 3.0 Unported License.


n = length(blobs);

[x y z] = size(dmat);

% inialise frequecy (pij) matrix
fmat = zeros(n,n);

% move 3x3x3 cube around space and calculate number of occupied cells each time

for iz=1:z-2
    for iy=1:y-2
        for ix=1:x-2
            % identify blob type of center cell
            pt = dmat(ix+1,iy+1,iz+1); 
            % get cube and place in vector
            cube=dmat(ix:ix+2,iy:iy+2,iz:iz+2);
            vcube=reshape(cube,1,27);
            % get frequency histogram for blob types
            fh = hist(vcube,blobs);
            % substract one for pt
            fh(pt) = fh(pt) - 1;
            % add these frequencies to the frequency matrix
            fmat(pt,:) = fmat(pt,:) + fh;
        end
    end
end

% calculate the pij's (relative frequencies) (Li & Reynolds, Eqn 4)
rfmat=zeros(n,n);

for i=1:n
    for j=1:n
        rfmat(i,j)=fmat(i,j)/sum(fmat(i,:));
    end 
end

% calculate EE (Li & Reynolds Eqn. 3)
ee = 0;

for i=1:n
    for j=1:n
        if rfmat(i,j) > 0
           ee = ee + rfmat(i,j)*log(rfmat(i,j));
        end
    end
end

ee = ee * -1;

% calculate emax (Li & Reynolds, Eqn 10)
emax = n* log(n);

% calculate contagion (Li & Reynolds, Eqn ?)
rc = 1 - ee/emax;

return
