function [ent_mat, stc]=stc(t)

% function [ent_mat, stc]=stc(t)
%
% Function to calculate the spatiotemporal complexity of a 3D data cube
% using a 3x3x3 moving window (overlapping in all dimensions)
%
% Input variables:
%
% t = the data matrix to be studied, should be binary
%
% Written by Lael Parrott, July 2003
%
% For details, see Parrott, L. (2005) Quantifying 
% the complexity of simulated spatiotemporal population 
% dynamics. Ecological Complexity, 2(2): 175-184.
%
% Distributed as part of the 3D Metrics Toolbox

% Licensed under a Creative Commons Attribution-NonCommercial-Share Alike
% 3.0 Unported License.

ent_mat=zeros(28,1);

[x y z] = size(t);

% move 3x3x3 cube around space and calculate number of occupied cells each time

for iz=1:z-2
    for iy=1:y-2
        for ix=1:x-2
            density=nnz(t(ix:ix+2,iy:iy+2,iz:iz+2));
            ent_mat(density+1)=ent_mat(density+1)+1;
        end
    end
end

% calculate entropy
entropy=0;

cells=sum(ent_mat);

for i=1:1:28
    freq=ent_mat(i)/cells;
    if freq==0
        entropy = entropy;
    else
        entropy = entropy + freq*log(freq);
    end 
end

% normalise the value
entropy = -1 * entropy;
logM=log(28);
stc=entropy/logM;

return



    




