function [ent_mat, stc]=stcvar_all(t,v1,v2,v3)

% function stcvar_all(t,v1,v2,v3)
%
% Function to calculate the spatiotemporal complexity of a 3D data cube
% using a v1 x v2 x v3 moving window (overlapping in all dimensions)
%
% This function is the same as stc.m except that the user can specify
% the size of the moving window.
%
% Input variables:
%
% t = the data matrix to be studied, should be binary
% v = size of moving window (v1=x dim; v2 = y dim; v3 = z dim)
%
% Written by Lael Parrott, July 2003; Modified May 2010
%
% For details, see Parrott, L. (2005) Quantifying 
% the complexity of simulated spatiotemporal population 
% dynamics. Ecological Complexity, in press.
%
% Distributed as part of the 3D Metrics Toolbox

% Licensed under a Creative Commons Attribution-NonCommercial-Share Alike
% 3.0 Unported License.

cubevol=v1*v2*v3;
ent_mat=zeros(cubevol+1,1);

[x y z] = size(t);

% move cube around space and calculate number of occupied cells each time

for iz=1:z-(v3-1)
    for iy=1:y-(v2-1)
        for ix=1:x-(v1-1)
            density=nnz(t(ix:ix+(v1-1),iy:iy+(v2-1),iz:iz+(v3-1)));
            ent_mat(density+1)=ent_mat(density+1)+1;
        end
    end
end

% calculate entropy
entropy=0;

cells=sum(ent_mat);

for i=1:1:cubevol+1
    freq=ent_mat(i)/cells;
    if freq==0
        entropy = entropy;
    else
        entropy = entropy + freq*log(freq);
    end 
end

% normalise the value
entropy = -1 * entropy;
logM=log(cubevol+1);
stc=entropy/logM;

return




    




